/* filename:    dsgeirf.g
** description: The program computes impulse response 
**              functions for the DSGE model
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include pathspec.g;


/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun  = "1";    /* "0" for draws from prior; "1" for draws from posterior */
nblock = 10000;  /* block size */
nuse   = 100;    /* use every nuse observation */
nresp  = 11;

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

/* Initialize Output files
*/
opath     = resupath $+ "\\mhrun" $+ mhrun;
chkdir    = chk_dir(opath);

odsgeirf    = opath $+ "\\dsgeirf";

create fhdsgeirf = ^odsgeirf with IRF, nresp*nvar*nirf, 8;

/* Initialization
*/ 

/* nblock  = 10000; */
eofloop = 0;
loopct  = 1;

"Generating IRFs...";

do until eofloop; 

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);
   npara   = cols(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   if ismiss(parasim) == 0;

   " ";
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);

   irfsim = zeros(nread,nresp*nvar*nirf);
   
   j = 1;
   do until j > rows(parasim);
   
      if fmod(j,nread/10)==0;".";; pause(1e-20); endif;
   
      impind = 1;
      do until impind > nvar;
      
         impact = zeros(nvar,1);
         impact[impind] = parasim[j,28+impind];
         
         yyirf = dsgeirf(parasim[j,.]',msel,nirf,impact);         
         irfsim[j,1+(impind-1)*nresp*nirf:impind*nresp*nirf] = vec(yyirf)';
         impind = impind+1;
      
      endo;
         
      j = j+1;
  endo;
      
  /* Write Output
  */
  wr = writer(fhdsgeirf,irfsim);

  endif;

  loopct = loopct + 1;

endo;

closeall fhpara, fhdsgeirf;

"DONE";

/********************************************************************
**   Procedure to compute Impulse Response Function
********************************************************************/
proc (1) = dsgeirf(para,msel,nirf,imp);
local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC, ssirf, yyirf, t,

      /*  - ordinary variables  */
      D_t,  F1_t, F2_t, H_t,  I_t,  K_tp1, M_tp1, MC_t, PI_t,
      PO_t, Q_t,  R_t,  Rk_t, UP_t, W_t,   X_t,   XI_t, Y_t, Ycal_t, Y_dot_t,
      BG_t, BGK_t, BGAM_t, BGAM_TEMP_t, BGQ_t, BU_t, BUP_t, BUPP_t, 
      BC_t, BCQ_t, BCK_t, BCQQ_t, BCKK_t, BCQK_t, mu_t,
      PItarget_t,
      /*  - jumpers */
      EPI_t, EPO_t, EF1_t, EXI_t, EF2_t, EUP_t, ERk_t, EBGAM_t, EBG_t, EBUP_t,
      EBGQ_t, EI_t, Emu_t,
      /*  - exogenous processes */
      GOV_t, CHI_t, MP_t, Z_t,
      /* additional variables to construct GDP */
      M_tp1lag, PIdm_t, PIgdp_t, Ygdp_t
      ;

/**********************************************************
**      System matrices for State-space
**********************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

/**********************************************************
**      Variable Indices for TT matrix
**********************************************************/

/** variable indices **/

D_t   = 1;    /* ordinary variables */
F1_t  = 2;
F2_t  = 3;
H_t   = 4;
I_t   = 5;
K_tp1 = 6;
M_tp1 = 7;
MC_t  = 8;
PI_t  = 9;
PO_t  = 10;
Q_t   = 11;
R_t   = 12;
Rk_t  = 13;
UP_t  = 14;
W_t   = 15;
X_t   = 16;
XI_t  = 17;
Y_t   = 18;
Ycal_t  = 19;
Y_dot_t = 20;
BG_t  = 21;
BGK_t = 22;
BGAM_t= 23;
BGAM_TEMP_t = 24;
BGQ_t = 25;
BU_t  = 26;
BUP_t = 27;
BUPP_t= 28; 
BC_t  = 29;
BCQ_t = 30;
BCK_t = 31;
BCQQ_t= 32;
BCKK_t= 33;
BCQK_t= 34;
EPI_t = 35;   /* period t expectations of t+1 terms */
EPO_t = 36;
EF1_t = 37;
EXI_t = 38;
EF2_t = 39;
EUP_t = 40;
ERk_t = 41;
EBGAM_t = 42;
EBG_t = 43;
EBUP_t= 44;
EBGQ_t= 45;
GOV_t = 46;   /* exogenous processes driven purely by shocks */
CHI_t = 47;
MP_t  = 48;
Z_t   = 49;
mu_t  = 50;
Emu_t = 51;
EI_t  = 52;
M_tp1lag = 53; /* additional variables to construct GDP */
PIdm_t   = 54;
PIgdp_t  = 55;
Ygdp_t   = 56;

PItarget_t = 58;

/**********************************************************
**      Create a new measurement equations (ZZ)
**********************************************************/

ZZ = zeros(11,rows(TT));

/* GDP */
ZZ[1,Ygdp_t] =  1;

/* Agg inflation */
ZZ[2,PIgdp_t] = 4;

/* CM Output */
ZZ[3,Y_t]=1;

/* DM Output */
ZZ[4,M_tp1lag] = 1; ZZ[4,PI_t] = -1;

/* DM Inflation */
ZZ[5,PIdm_t] = 4; 

/* Relative price DM/CM */
ZZ[6,M_tp1lag] = 1; ZZ[6,Q_t] = -1; ZZ[6,PI_t] = -1;

/* Interest Rates */
ZZ[7,R_t] = 4;

/* Money balances */
ZZ[8,M_tp1] = 1;

/* Target Inflation */
ZZ[9,PItarget_t] = 4; 

/* CM Inflation */
ZZ[10,PI_t] = 4; 

/* Velocity */
ZZ[11,M_tp1] = 1; ZZ[11,Ygdp_t] = -1;

/* Compute IRFs */

ssirf = zeros(nirf,cols(ZZ));
yyirf = zeros(nirf,rows(ZZ));

ssirf[1,.] = (RR*imp)';
yyirf[1,.] = ssirf[1,.]*ZZ';
t = 2;
do until t > nirf;
   ssirf[t,.] = (TT*ssirf[t-1,.]')';
   yyirf[t,.] = ssirf[t,.]*ZZ';
   t = t+1;
endo;

eofirf:

retp(yyirf);
endp;
